rm(list=ls())
require(zoo)
require(ggplot2)
require(rugarch)
require(knitr)


source("LoadFundData.R")
# source("LoadWigData.R")

par(mfrow=c(2,1), cex = 0.7, bty="l")
plot(P, main="level", xlab="", ylab="")
plot(r, main="returns", xlab="", ylab="")

########################################
# Stress-test settings                 #
########################################
 
T     <- length(r)
N     <- T                               
r     <- tail(r,N)                       
R     <- coredata(r)

##############################################################
# 1. Historical scenario                                     #
# Worst return in window w_length days                       #
##############################################################

# window length
w_length = 250             

# rolling minimum 
MAmin     <- rollapply(P, width=w_length, min, by=1, align="left")
MAdecline <- MAmin/P[1:length(MAmin)]-1           

N0     <- which.min(MAdecline)
MAdecline[N0]

plot(MAdecline)
abline(v=index(MAdecline[N0]),col=2, lwd=3)

ScHist <- min(MAdecline)          # maximum decline during a year

# dates of extreme decline 
N1        <- N0 + which.min(P[N0:(N0+w_length-1)]) - 1
P[c(N0,N1)]

##############################################################
# 2A. Standard hypothetical scenario                         #
##############################################################

# Portfolio structure
A1 = 1       # domestic stocks
A2 = 0       # foreign stocks
A3 = 0       # gov. bonds, 5-years
A4 = 0       # corporate bonds, 3 -years
A5 = 0       # commodities
A = c(A1,A2,A3,A4,A5)

# Stage 1: sensitivity analysis vs risk factors

RF1 = sum(c(1, 1, 0, 0, 0)*A)    # percentage reaction to 1% increase in stock indices (at home and abroad) 
RF2 = sum(c(0, 0,-5,-3, 0)*A)    # percentage reaction to 100bp. upward shift of enire yield curve
RF3 = sum(c(0, 1, 0, 0, 1)*A)    # percentage reaction to 1% exchange rate depreciation 
RF4 = sum(c(0, 0, 0,-3, 0)*A)    # percentage reaction to 100bp. credit spread increase 
RF5 = sum(c(1, 0, 0, 0, 1)*A)    # percentage reaction to 1% increase in commodity prices 

# Stage 2: scenario analysis  
S1 = -abs(RF1*10);  # increase and decrease in equity index values by 10 percent
S2 = -abs(RF2);     # parallel yield curve shifts of 100 basis points up and down
S3 = -abs(RF3*20);  # increase and decrease in the exchange value of foreign currencies by 20 percent 
S4 = -abs(RF4);     # increase and decrease in cedit spreads by 100 basis points.

ScHipA <- S1+S2+S3+S4;
ScHipA                       # hypothtical stress scenario 

##############################################################
# 2B. Unique stress scenario                                 #
##############################################################

# Lets assume that PL is develping country and a scenario of global risk aversion:
# 1. capital outflow from developing countries and depreciation of their currencies by 10%
# 2. stock prices fall by 15%
# 3. yield curve fall by 100 pb in developed countries 
# 4. yield curve fall by 50 pb in developing countries
# 5. commodity price decline by 25%

# Impact on portfolio
ScHipB <- RF1*(-15) + RF2*(-1) + RF3*10 + RF4*0.5 + RF5*(-25)
ScHipB

##########################
# 3. stressed VaR, S-VaR #
##########################

p = 0.01   # tolerance level
H = 250    # horizon
w_length  <- 250           # window length
q         <- 0.05          # quantile for (stressed) statistics     

# assume that r~N(mu,sig^2)

# standard VaR/ES  
N    <- 1250               
R    <- tail(R,N)
# r    <- tail(r,N)

# H-horizon VaR (SQRT method)
m0   <- mean(R)*H
s0   <- sd(R)*sqrt(H)
S_VaR0    <- m0 + qnorm(p)*s0            
S_ES0     <- m0 - s0*dnorm(qnorm(p))/p   

# stressed mean
MAmean    <- rollapply(r, width=w_length, mean, by=1, align="right")
m1        <- quantile(MAmean,q, names=FALSE)*H
s1        <- s0
kable(c(m0,m1),digits=3, col.names = "stresses mean")
S_VaR1    <- m1 + qnorm(p)*s1            # stressed VaR 
S_ES1     <- m1 - s1*dnorm(qnorm(p))/p   # stressed Expected shortfall

# stressed std.
MAsd      <- rollapply(r, width=w_length, sd, by=1, align="right")
m2        <- m0
s2        <- quantile(MAsd,1-q, names=FALSE)*sqrt(H)
kable(c(s0,s2),digits=3, col.names = "stresses std.")
S_VaR2    <- m2 + qnorm(p)*s2            # stressed VaR 
S_ES2     <- m2 - s2*dnorm(qnorm(p))/p   # stressed Expected shortfall

# stressed mean and std.
m3        <- m1
s3        <- s2
S_VaR3    <- m3 + qnorm(p)*s3            # stressed VaR 
S_ES3     <- m3 - s3*dnorm(qnorm(p))/p   # stressed Expected shortfall

kable(100*c(S_VaR0,S_VaR1,S_VaR2,S_VaR3), col.names = "SVaR comparison")
kable(100*exp(c(S_VaR0,S_VaR1,S_VaR2,S_VaR3)), col.names = "100PLN value")

